@@ -6,34 +6,37 @@
 
 #' Returns TRUE if value is not NA
 #'
-#' This is the inverse of \code{\link[base]{is.na}} if it is
-#' used on a atomic element. This is a convenience function meant
-#' to be used as a predicate in an \code{\link{assertr}} assertion.
+#' This is the inverse of \code{\link[base]{is.na}}. This is a convenience
+#' function meant to be used as a predicate in an \code{\link{assertr}}
+#' assertion.
 #'
-#' @param x A single atomic value
+#' @param x A R object that supports \link{is.na} an \link{is.nan}
 #' @param allow.NaN A logical indicating whether NaNs should be allowed
 #'        (default FALSE)
-#' @return TRUE if x is not NA, FALSE otherwise
-#' @seealso \code{\link{is.na}}
+#' @return A vector of the same length that is TRUE when the element is
+#' not NA and FALSE otherwise
+#' @seealso \code{\link{is.na}} \code{\link{is.nan}}
 #' @examples
 #' not_na(NA)
 #' not_na(2.8)
 #' not_na("tree")
+#' not_na(c(1, 2, NA, 4))
 #'
 #' @export
 not_na <- function(x, allow.NaN=FALSE){
-  if(length(x)>1)            stop("not_na must be called with single element")
-  if(is.null(x))             stop("not_na must be called with single element")
-  if(allow.NaN && is.nan(x)) return(TRUE)
-  if(is.na(x))               return(FALSE)
-  return(TRUE)
+  if(is.null(x))    stop("not_na must be called on non-null object")
+  if(allow.NaN)     return((!is.na(x)) | is.nan(x))
+  return(!is.na(x))
 }
+# so assert function knows to vectorize the function for
+# substantial speed increase
+comment(not_na) <- "assertr/vectorized"
 
 
 #' Creates bounds checking predicate
 #'
-#' This function returns a predicate function that will take a single
-#' numeric value and return TRUE if the value is within the bounds set.
+#' This function returns a predicate function that will take a numeric value
+#' or vector and return TRUE if the value(s) is/are within the bounds set.
 #' This does not actually check the bounds of anything--it only returns
 #' a function that actually does the checking when called with a number.
 #' This is a convenience function meant to return a predicate function to
@@ -48,8 +51,8 @@ not_na <- function(x, allow.NaN=FALSE){
 #' @param allow.na A logical indicating whether NAs (including NaNs)
 #'        should be permitted (default TRUE)
 #'
-#' @return A function that takes one numeric and returns TRUE
-#'         if the value is within the bounds defined by the
+#' @return A function that takes numeric value or numeric vactor and returns
+#'         TRUE if the value(s) is/are within the bounds defined by the
 #'         arguments supplied by \code{within_bounds} and FALSE
 #'         otherwise
 #'
@@ -71,7 +74,7 @@ not_na <- function(x, allow.NaN=FALSE){
 #' ## this is meant to be used as a predicate in an assert statement
 #' assert(mtcars, within_bounds(4,8), cyl)
 #'
-#' ## or in a pipeline, like this was meant for
+#' ## or in a pipeline
 #'
 #' library(magrittr)
 #'
@@ -86,27 +89,28 @@ within_bounds <- function(lower.bound, upper.bound,
     stop("bounds must be numeric")
   if(lower.bound >= upper.bound)
     stop("lower bound must be strictly lower than upper bound")
-  function(x){
-    if(length(x)>1)      stop("bounds must be checked on a single element")
-    if(is.null(x))       stop("bounds must be checked on a single element")
+  fun <- function(x){
+    if(is.null(x))       stop("bounds must be checked on non-null element")
     if(!is.numeric(x))   stop("bounds must only be checked on numerics")
-    if(is.na(x)){
-      if(allow.na)    return(TRUE)
-      if(!allow.na)   return(FALSE)
-    }
     lower.operator <- `>=`
     if(!include.lower) lower.operator <- `>`
     upper.operator <- `<=`
     if(!include.upper) upper.operator <- `<`
-    if(lower.operator(x, lower.bound) && upper.operator(x, upper.bound))
-      return(TRUE)
-    return(FALSE)
+    if(allow.na){
+      return((lower.operator(x, lower.bound) &
+                upper.operator(x, upper.bound)) | is.na(x))
+    }
+    return((lower.operator(x, lower.bound) &
+              upper.operator(x, upper.bound)) & !(is.na(x)))
   }
+  comment(fun) <- "assertr/vectorized"
+  return(fun)
 }
 # so, this function returns a function to be used as argument to another
 # function
 
 
+
 #' Returns TRUE if value in set
 #'
 #' This function returns a predicate function that will take a single
@@ -188,7 +192,7 @@ in_set <- function(..., allow.na=TRUE){
 #' @return A function that takes a vector and returns a
 #'         \code{\link{within_bounds}} predicate based on the standard deviation
 #'         of that vector.
-#'
+#' @seealso \code{\link{within_n_mads}}
 #' @examples
 #' test.vector <- rnorm(100, mean=100, sd=20)
 #'
@@ -204,7 +208,7 @@ in_set <- function(..., allow.na=TRUE){
 #' # because, by default, within_bounds() will accept
 #' # NA values. If we want to reject NAs, we have to
 #' # provide extra arguments to this function
-#' within_n_sds(2, allow.na=FALSE)(test.vector)(as.numeric(NA))  # returns TRUE
+#' within_n_sds(2, allow.na=FALSE)(test.vector)(as.numeric(NA))  # returns FALSE
 #'
 #' # or in a pipeline, like this was meant for
 #'
@@ -230,3 +234,72 @@ within_n_sds <- function(n, ...){
 }
 
 
+#' Return a function to create robust z-score checking predicate
+#'
+#' This function takes one argument, the number of median absolute
+#' deviations within which to accept a particular data point. This is
+#' generally more useful than its sister function \code{\link{within_n_sds}}
+#' because it is more robust to the presence of outliers. It is therefore
+#' better suited to identify potentially erroneous data points.
+#'
+#' As an example, if '2' is passed into this function, this will return
+#' a function that takes a vector and figures out the bounds of two
+#' median absolute deviations (MADs) from the median. That function will then
+#' return a \code{\link{within_bounds}} function that can then be applied
+#' to a single datum. If the datum is within two MADs of the median of the
+#' vector given to the function returned by this function, it will return TRUE.
+#' If not, FALSE.
+#'
+#' This function isn't meant to be used on its own, although it can. Rather,
+#' this function is meant to be used with the \code{\link{insist}} function to
+#' search for potentially erroneous data points in a data set.
+#'
+#' @param n The number of median absolute deviations from the median
+#'        within which to accept a datum
+#' @param ... Additional arguments to be passed to \code{\link{within_bounds}}
+#'
+#' @return A function that takes a vector and returns a
+#'         \code{\link{within_bounds}} predicate based on the MAD
+#'         of that vector.
+#' @seealso \code{\link{within_n_sds}}
+#' @examples
+#' test.vector <- rnorm(100, mean=100, sd=20)
+#'
+#' within.one.mad <- within_n_mads(1)
+#' custom.bounds.checker <- within.one.mad(test.vector)
+#' custom.bounds.checker(105)     # returns TRUE
+#' custom.bounds.checker(40)      # returns FALSE
+#'
+#' # same as
+#' within_n_mads(1)(test.vector)(40)    # returns FALSE
+#'
+#' within_n_mads(2)(test.vector)(as.numeric(NA))  # returns TRUE
+#' # because, by default, within_bounds() will accept
+#' # NA values. If we want to reject NAs, we have to
+#' # provide extra arguments to this function
+#' within_n_mads(2, allow.na=FALSE)(test.vector)(as.numeric(NA))  # returns FALSE
+#'
+#' # or in a pipeline, like this was meant for
+#'
+#' library(magrittr)
+#'
+#' iris %>%
+#'   insist(within_n_mads(5), Sepal.Length)
+#'
+#' @export
+within_n_mads <- function(n, ...){
+  if(!is.numeric(n) || length(n)!=1 || n<=0){
+    stop("'n' must be a positive number")
+  }
+  function(a.vector){
+    if(!is.vector(a.vector) || !is.numeric(a.vector))
+      stop("argument must be a numeric vector")
+    dmad <- mad(a.vector, na.rm=TRUE)
+    dmed <- median(a.vector, na.rm=TRUE)
+    if(is.na(dmad)) stop("MAD of vector is NA")
+    if(is.na(dmed)) stop("median of vector is NA")
+    within_bounds((dmed-(n*dmad)), (dmed+(n*dmad)), ...)
+  }
+}
+
+
