@@ -20,3 +20,10 @@ not.empty.p <- function(x) if(x=="") return(FALSE)
 ## ------------------------------------------------------------------------
 seven.digit.p <- function(x) nchar(x)==7
 
+## ----perl=FALSE----------------------------------------------------------
+example.data <- data.frame(x=c(8, 9, 6, 5, 9, 5, 6, 7,
+                             8, 9, 6, 5, 5, 6, 7),
+                         y=c(82, 91, 61, 49, 40, 49, 57,
+                             74, 78, 90, 61, 49, 51, 62, 68))
+(example.data)
+
